/*
	25.3.2006 Louis Thoumin, 4D SA
 
	Handle movie in an offscreen area. 
	Example Import mp3 in background. Can be usefull to retrieve information about length etc...
*/


#include "QTPlayextern.h"
#include "QTPlay.h"
#include "QTFile.h"


	
/* Some documentation...
http://developer.apple.com/documentation/QuickTime/APIREF/getmovietimebase.htm#//apple_ref/doc/uid/TP40001187-DontLinkChapterID_1-GetMovieTimeBase
http://developer.apple.com/documentation/QuickTime/APIREF/MovieControllerActions.htm#//apple_ref/doc/constant_group/Movie_Controller_Actions
http://developer.apple.com/documentation/QuickTime/APIREF/AlphabeticalIndexofFunctions.htm
*/

/*
	Create an offscreen area
	QTNewOffscreenArea (
						AreaRef		<-  Long
	)
		error code		<-	Long
*/
void QTNewOffscreenArea ( PA_PluginParameters params )
{
	QTArea_Init ( params , kIsNotExternalArea ) ;
}


/* Remove the movie from  an offscreen area
		QTDisposeMovie (
			AreaRef			->	Long
		)
			error code		<-	Long
*/

void QTDisposeMovie ( PA_PluginParameters params )
{
	enum {
		kparam_areaRef	= 1
	};
	long		err					= noErr;
	MovieStuff* gMovieStuff;

	gMovieStuff = 0L;
	gMovieStuff = (MovieStuff*)PA_GetLongParameter ( params , kparam_areaRef );
	if ( gMovieStuff != 0L )
	{
	  if (gMovieStuff->Tag != 'TMQT')
			err = kErr_BadKindArea;
	  else
		{
		if ( gMovieStuff->IsExternalAreaInForm == kIsNotExternalArea )
		{
			if ( (gMovieStuff->movieOpened) == TRUE )
			{
				if (gMovieStuff->theMovie)
					DisposeMovie(gMovieStuff->theMovie);
						
				gMovieStuff->theMovie = 0L;
				gMovieStuff->movieOpened = false;
			}
		}
		else
		{
			if (gMovieStuff->theMC)
				DisposeMovieController(gMovieStuff->theMC);
			if (gMovieStuff->theMovie)
				DisposeMovie(gMovieStuff->theMovie);	
			if (gMovieStuff->filename)
				free(gMovieStuff->filename);
			gMovieStuff->theMovie = 0L;
			gMovieStuff->movieOpened = false;	
		}
		}
	}
	else
		err = kErr_MemError;

	PA_ReturnLong ( params , err );
 
}

/* Destroy an offscreen area
		QTDeleteOffscreenArea (
			AreaRef			->	Long
		)
			error code		<-	Long
*/
void QTDeleteOffscreenArea ( PA_PluginParameters params )				// 05.04.2006 Louis Thoumin
{
	enum {
		kparam_areaRef	= 1
	};
	long		err					= noErr;
	MovieStuff* gMovieStuff;

	gMovieStuff = 0L;
	gMovieStuff = (MovieStuff*)PA_GetLongParameter ( params , kparam_areaRef );
	if ( gMovieStuff != 0L )
	{
		if ( gMovieStuff->IsExternalAreaInForm == kIsNotExternalArea )
		{
			if (gMovieStuff->movieOpened)
				QTDisposeMovie ( params );
			
				// free the memory
				free ( (void*)gMovieStuff );
				PA_SetLongParameter ( params , kparam_areaRef , 0 );
		}
		else
			err = kErr_BadKindArea;
	}
	else
		err = kErr_BadKindArea;
	
	PA_ReturnLong ( params , err );
}


/*
 INTERNAL USE - MAC ONLY
 We are able to open a fullpath > 255, and unicode.
 */
#if VERSIONMAC
	// NewPath		can be NULL		Encoding MacRoman
	// newPathUTF8  can be NULL		Encoding UTF8
	OSErr build_4DPath_2_PosixPath (char* Pfad , char *NewPath , UInt8 *newPathUTF8 , CFIndex buffSize  )
	{
		OSErr		err			= noErr;
		CFStringRef filePath	= NULL;
		CFURLRef	url			= NULL;
		CFStringRef altpath		= NULL ;
		//CFIndex		theLen;
		TextEncoding			myTextEncoding = kCFStringEncodingMacRoman;  
		
		// filePath = CFStringCreateWithCString ( NULL, Pfad, myTextEncoding);
		// filePath = CFStringCreateWithCString ( NULL, Pfad, myTextEncoding); // Not really good, because of NULL in Unicode
		
		filePath = CFStringCreateWithBytes ( kCFAllocatorDefault,
											 (const UInt8*) Pfad,
											 (CFIndex) strlen(Pfad),
											 myTextEncoding, /* CFStringEncoding encoding,*/
											 false);
		
		if ( filePath != NULL )
		{
			url = CFURLCreateWithFileSystemPath(NULL, filePath, kCFURLHFSPathStyle, false);
			if ( url != NULL )
			{
				altpath = CFURLCopyFileSystemPath(url, kCFURLPOSIXPathStyle);
				if ( altpath != NULL )
				{
					if ( newPathUTF8 != NULL)
						CFStringGetCString (altpath, (char*)newPathUTF8, buffSize, kCFStringEncodingUTF8 );
					
					if ( NewPath != NULL)
						CFStringGetCString (altpath, NewPath, buffSize, kCFStringEncodingMacRoman );				
				}
			}
		}
		
		if ( filePath != NULL )
			CFRelease(filePath);
		else
			err = kErr_MemError;
		
		if ( url != NULL )
			CFRelease(url);
		else
			err = kErr_MemError;
		
		if ( altpath != NULL )
			CFRelease(altpath);
		else
			err = kErr_MemError;
		
		return ( err );
		
	}
#endif



long build_FSSpecFrompath ( char* thePath, FSSpec *myFSSpec)
{
	long		err					= noErr;
	long		pathLen				= 0;
#if VERSIONMAC
	Boolean		isDirectory		= false;
	UInt8		*NewPath_utf8   = NULL;
	FSRef		FSRef;
#endif
	
	
#if VERSIONWIN
			err = NativePathNameToFSSpec ( thePath , myFSSpec , kErrorIfFileNotFound );
#else			
			pathLen=strlen(thePath) * 3;
			NewPath_utf8 = (UInt8*) malloc ( pathLen );  // to have enough space for diacritical chars
			if ( NewPath_utf8 != NULL ) 
				err = build_4DPath_2_PosixPath (thePath , 0L , NewPath_utf8 , pathLen );
			else
				err = kErr_MemError;
			
			if (( NewPath_utf8 != NULL ) && ( err == noErr ))
				err = FSPathMakeRef ( NewPath_utf8 , &FSRef, &isDirectory);
			
			if ( err == noErr )
				err = FSGetCatalogInfo ( &FSRef , kFSCatInfoNone ,
										 NULL, NULL,  myFSSpec,  NULL );
	
			if ( NewPath_utf8 != NULL )
				free ( NewPath_utf8);
#endif
		
		return err;
	
}

